import 'package:yuro_cli/command/command.dart';
import 'package:yuro_cli/core/core.dart';

class Upgrade extends Command {
  @override
  String get name => 'upgrade';

  @override
  String get help => 'Upgrade this CLI.';

  @override
  ArgParser createArgParser() => ArgParser();

  @override
  Future<void> parser(List<String> arguments) async {
    var remoteVersion = await PubUtil.getRemoteVersion('yuro_cli');
    var nativeVersion = await YamlUtil.getNativeVersion();
    if (nativeVersion == remoteVersion) {
      Logger.i('the latest version is already installed');
      return;
    } else {
      Logger.i('the current version is $nativeVersion and the latest version is $remoteVersion. upgrade...');

      var lockFile = File(join(flutterPubPath, 'global_packages/yuro_cli/pubspec.lock'));
      ProcessResult result;
      if (lockFile.existsSync()) {
        result = await run('flutter', ['pub', 'global', 'activate', 'yuro_cli']);
      } else {
        result = await run('pub', ['global', 'activate', 'yuro_cli']);
      }
      if (result.exitCode != 0) {
        Logger.e(' yuro_cli upgrade failed, exitCode: ${result.exitCode}');
      } else {
        Logger.s('yuro_cli has been updated to the latest version.');
      }
    }
  }
}
